@@ -4,24 +4,31 @@ Tests for the django-reversion API.
 These tests require Python 2.5 to run.
 """
 
-from __future__ import with_statement
+from __future__ import unicode_literals
 
 import datetime, os
 
 from django.db import models
 from django.test import TestCase
 from django.core.management import call_command
+from django.core.exceptions import ImproperlyConfigured
 from django.conf import settings
-from django.conf.urls.defaults import *
+from django.conf.urls import url, patterns, include
 from django.contrib import admin
-from django.contrib.auth.models import User
+try:
+    from django.contrib.auth import get_user_model
+except ImportError: # django < 1.5
+    from django.contrib.auth.models import User
+else:
+    User = get_user_model()
 from django.utils.decorators import decorator_from_middleware
 from django.http import HttpResponse
 from django.utils.unittest import skipUnless
+from django.utils.encoding import force_text, python_2_unicode_compatible
 
 import reversion
 from reversion.revisions import RegistrationError, RevisionManager
-from reversion.models import Revision, Version, VERSION_ADD, VERSION_CHANGE, VERSION_DELETE
+from reversion.models import Revision, Version
 from reversion.middleware import RevisionMiddleware
 
 
@@ -40,13 +47,14 @@ class UTC(datetime.tzinfo):
     def dst(self, dt):
         return ZERO
 
+@python_2_unicode_compatible
 class ReversionTestModelBase(models.Model):
 
     name = models.CharField(
         max_length = 100,
     )
     
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
     class Meta:
@@ -64,7 +72,7 @@ str_pk_gen = 0;
 def get_str_pk():
     global str_pk_gen
     str_pk_gen += 1;
-    return str(str_pk_gen)
+    return force_text(str_pk_gen)
     
     
 class ReversionTestModel2(ReversionTestModelBase):
@@ -80,8 +88,8 @@ class ReversionTestModel1Proxy(ReversionTestModel1):
 
     class Meta:
         proxy = True
-        
-        
+
+
 class RevisionMeta(models.Model):
 
     revision = models.OneToOneField(Revision)
@@ -227,21 +235,6 @@ class InternalsTest(RevisionTestBase):
             pass
         self.assertEqual(Revision.objects.count(), 1)
         self.assertEqual(Version.objects.count(), 4)
-        
-    def testCorrectVersionFlags(self):
-        self.assertEqual(Version.objects.filter(type=VERSION_ADD).count(), 4)
-        self.assertEqual(Version.objects.filter(type=VERSION_CHANGE).count(), 0)
-        self.assertEqual(Version.objects.filter(type=VERSION_DELETE).count(), 0)
-        with reversion.create_revision():
-            self.test11.save()
-        self.assertEqual(Version.objects.filter(type=VERSION_ADD).count(), 4)
-        self.assertEqual(Version.objects.filter(type=VERSION_CHANGE).count(), 1)
-        self.assertEqual(Version.objects.filter(type=VERSION_DELETE).count(), 0)
-        with reversion.create_revision():
-            self.test11.delete()
-        self.assertEqual(Version.objects.filter(type=VERSION_ADD).count(), 4)
-        self.assertEqual(Version.objects.filter(type=VERSION_CHANGE).count(), 1)
-        self.assertEqual(Version.objects.filter(type=VERSION_DELETE).count(), 1)
 
 
 class ApiTest(RevisionTestBase):
@@ -633,6 +626,13 @@ def error_revision_view(request):
     raise Exception("Foo")
 
 
+# A dumb view that has two revision middlewares.
+@revision_middleware_decorator
+@revision_middleware_decorator
+def double_middleware_revision_view(request):
+    raise Exception("Foo")
+
+
 site = admin.AdminSite()
 
 
@@ -646,13 +646,14 @@ class ParentTestAdminModel(models.Model):
         app_label = "auth"  # Hack: Cannot use an app_label that is under South control, due to http://south.aeracode.org/ticket/520
 
 
+@python_2_unicode_compatible
 class ChildTestAdminModel(ParentTestAdminModel):
 
     child_name = models.CharField(
         max_length = 200,
     )
     
-    def __unicode__(self):
+    def __str__(self):
         return self.child_name
     
     class Meta:
@@ -662,16 +663,52 @@ class ChildTestAdminModel(ParentTestAdminModel):
 class ChildTestAdminModelAdmin(reversion.VersionAdmin):
 
     pass
-    
-    
+
+
 site.register(ChildTestAdminModel, ChildTestAdminModelAdmin)
 
 
+class InlineTestParentModel(models.Model):
+    name = models.CharField(max_length=100,)
+
+    def __str__(self):
+        return self.name
+
+    class Meta:
+        app_label = "auth"  # Hack: Cannot use an app_label that is under South control, due to http://south.aeracode.org/ticket/520
+
+
+class InlineTestChildModel(models.Model):
+    parent = models.ForeignKey(InlineTestParentModel, related_name='children')
+    name = models.CharField(max_length=100)
+
+    def __str__(self):
+        return self.name
+
+    class Meta:
+        app_label = "auth"  # Hack: Cannot use an app_label that is under South control, due to http://south.aeracode.org/ticket/520
+
+
+class InlineTestChildModelInline(admin.TabularInline):
+    model = InlineTestChildModel
+    fk_name = 'parent'
+    extra = 0
+    verbose_name = 'Child'
+    verbose_name_plural = 'Children'
+
+
+class InlineTestParentModelAdmin(reversion.VersionAdmin):
+    inlines = (InlineTestChildModelInline, )
+site.register(InlineTestParentModel, InlineTestParentModelAdmin)
+
+
 urlpatterns = patterns("",
 
     url("^success/$", save_revision_view),
     
     url("^error/$", error_revision_view),
+
+    url("^double/$", double_middleware_revision_view),
     
     url("^admin/", include(site.get_urls(), namespace="admin")),
 
@@ -696,6 +733,9 @@ class RevisionMiddlewareTest(ReversionTestBase):
         self.assertEqual(Revision.objects.count(), 0)
         self.assertEqual(Version.objects.count(), 0)
 
+    def testRevisionMiddlewareErrorOnDoubleMiddleware(self):
+        self.assertRaises(ImproperlyConfigured, lambda: self.client.get("/double/"))
+
 
 class VersionAdminTest(TestCase):
 
@@ -787,7 +827,73 @@ class VersionAdminTest(TestCase):
             "child_name": "child instance1 version4",
         })
         obj = ChildTestAdminModel.objects.get(id=obj_pk)
-        
+
+
+    def createInlineObjects(self, should_delete):
+        # Create an instance via the admin without a child.
+        response = self.client.post("/admin/auth/inlinetestparentmodel/add/", {
+            "name": "parent version1",
+            "children-TOTAL_FORMS": "0",
+            "children-INITIAL_FORMS": "0",
+            # "children-0-name": "child version 1",
+            "_continue": 1,
+            })
+        self.assertEqual(response.status_code, 302)
+        parent_pk = response["Location"].split("/")[-2]
+        parent = InlineTestParentModel.objects.get(id=parent_pk)
+
+        # Update  instance via the admin to add a child
+        response = self.client.post("/admin/auth/inlinetestparentmodel/%s/" % parent_pk, {
+            "name": "parent version1",
+            "children-TOTAL_FORMS": "1",
+            "children-INITIAL_FORMS": "0",
+            "children-0-name": "child version 1",
+            "_continue": 1,
+            })
+        self.assertEqual(response.status_code, 302)
+        children = InlineTestChildModel.objects.filter(parent=parent_pk)
+        self.assertEqual(children.count(), 1)
+
+        # get list of versions
+        version_list = reversion.get_for_object(parent)
+        self.assertEqual(len(version_list), 2)
+
+        # check if reversion page has the checkbox for the inline checked
+        response = self.client.get("/admin/auth/inlinetestparentmodel/%s/history/%s/" %
+                                   (parent_pk, version_list[1].id))
+        self.assertEqual(response.status_code, 200)
+        if should_delete:
+            self.assertContains(response, '<input checked="checked" id="id_children-0-DELETE" name="children-0-DELETE" type="checkbox" />') # this is crude
+        else:
+            self.assertNotContains(response, '<input checked="checked" id="id_children-0-DELETE" name="children-0-DELETE" type="checkbox" />') # this is crude
+
+        # don't actually submit a post since the values we submit would be from the test, not what the admin defaults
+
+
+    @skipUnless('django.contrib.admin' in settings.INSTALLED_APPS,
+                "django.contrib.admin not activated")
+    def testInlineAdmin(self):
+        self.assertTrue(reversion.is_registered(InlineTestParentModel))
+
+        # make sure model is following the child FK
+        self.assertTrue('children' in reversion.get_adapter(InlineTestParentModel).follow)
+
+        self.createInlineObjects(True)
+
+        # unregister model
+        reversion.unregister(InlineTestParentModel)
+        self.assertFalse(reversion.is_registered(InlineTestParentModel))
+
+        # re-register without following
+        reversion.register(InlineTestParentModel)
+        self.assertTrue(reversion.is_registered(InlineTestParentModel))
+
+        # make sure model is NOT following the child FK
+        self.assertFalse('children' in reversion.get_adapter(InlineTestParentModel).follow)
+
+        self.createInlineObjects(False)
+
+
     def tearDown(self):
         self.client.logout()
         self.user.delete()
@@ -832,14 +938,4 @@ class PatchTest(RevisionTestBase):
     def tearDown(self):
         super(PatchTest, self).tearDown()
         del self.version1
-        del self.version2
-
-
-# Import the deprecated tests.
-from reversion import tests_deprecated
-
-for name, value in vars(tests_deprecated).iteritems():
-    if isinstance(value, type) and issubclass(value, TestCase):
-        globals()[name] = value
-del name
-del value
+        del self.version2
\ No newline at end of file
